import React, {Component} from 'react';
import  {connect} from 'react-redux';
import TimePicker from '../components/timepicker-component';
import * as actions from '../action-creators';


function parentsHaveClassName(element, className) {
    var parent = element;
    while (parent) {
        if (parent.className && parent.className.indexOf(className) > -1) return true;

        parent = parent.parentNode;
    }

    return false;
}


class TimePickerContainer extends React.Component {

    componentWillMount() {
        console.log('componentWillMount', this.props)
        let self = this;
        document.addEventListener("click", (e) => {
            if (!parentsHaveClassName(e.target, "time-picker"))
                self.props.hideClock();
        });
    }

    componentWillUnmount() {
        let self = this;
        document.removeEventListener("click", (e) => {
            if (!parentsHaveClassName(e.target, "time-picker"))
                self.props.hideClock();
        });
    }

    render() {
        console.log(this.props)
        return (
            <TimePicker {...this.props} />
        );
    }

}

const mapStateToProps = (store) => {
    console.log(store)
    return {
        timepicker: store.timepicker
    };
}

const mapDispatchToProps = (dispatch) => {
    return ({
        hideClock: () => {
            console.log('calling jhide ')
            dispatch(actions.hideClock())
        },
        showClock: (trigger) => {
            let rect = trigger.getBoundingClientRect(),
                isTopHalf = rect.top > window.innerHeight / 2;
            dispatch(actions.showClock({

                top: isTopHalf ? rect.top + window.scrollY - this.props.CLOCK_SIZE - 3 : rect.top + trigger.clientHeight + window.scrollY + 3,
                left: rect.left

            }))
        }
    })
}
export default connect(mapStateToProps, mapDispatchToProps)(TimePickerContainer);


